#
# fishing.R, 20 Aug 23

library("colorspace")
library("plyr")
library("stringr")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


find_methods=function(df)
{
# Now the methods are located, summarise the features of interest
method_summary=function(df)
   {
   df$m_end=max(df$m_end) # common method end
   df$num_locals=nrow(df)
   df$num_lines=df$m_end[1]-df$m_start[1]
   df$span_frac=df$range/df$num_lines # Fraction of possible span
   df$begin_ord=1:nrow(df) # order of declaration

   return(df)
   }

# Assume that a method consists of a sequence of overlepping
# variable definition/last use.
# Sort varaibles by begin_line, then merge any following variables
# whose begin_line is less-equal that current last line
if (nrow(df) == 1)
   {
   df$begin_ord=1
   return(df)
   }
# print(df$path[1])
df=df[order(df$begin_line, decreasing=FALSE), ]
df$m_start=NA
df$m_end=NA

df$m_start[1]=df$begin_line[1]
df$m_end[1]=df$end_line[1]
for (lv in 2:nrow(df))
   {
   if (df$begin_line[lv] <= df$m_end[lv-1]) # within range of last variable?
      {
      df$m_start[lv]=df$m_start[lv-1]
      df$m_end[lv]=max(df$m_end[lv-1], df$end_line[lv])
      } else # start a new method
      {
      df$m_start[lv]=df$begin_line[lv]
      df$m_end[lv]=df$end_line[lv]
      }
   }

mdf=ddply(df, .(m_start), method_summary)

return(mdf)
}


file_summary=function(df)
{
# df$begin_ord=rank(df$begin_line) # ties get given the average value
# df$end_ord=rank(df$end_line)
# The following ranking, treating ties as different, gives a much better fit
df$begin_ord[order(df$begin_line, decreasing=TRUE)]=1:nrow(df)
df$end_ord[order(df$end_line, decreasing=TRUE)]=1:nrow(df)
df$num_locals=nrow(df)
df$num_lines=max(df$end_line)-min(df$begin_line)

return(df)
}


vs=read.csv("categorized_local_vars.txt.xz", as.is=TRUE, sep="\t")

vs$num_words=1+str_count(vs$details, ",") # actually the number of linguistic subcomponents

# file_info=ddply(vs, .(path), file_summary)

vs=ddply(vs, .(path), find_methods)

vs50=subset(vs, num_locals < 50) # Remove automatically generated methods


method_info=ddply(vs50, .(path, m_start), function(df) return(df[1, ]))

# Are variables declared earlier used over more of the code?
plot(vs50$begin_ord, vs50$span_frac)
smoothScatter(vs50$begin_ord, vs50$span_frac,
	xaxs="i", yaxs="i",
	xlim=c(1, 50), ylim=c(0, 1))


vs5_50=subset(vs50, num_lines > 5)
plot(count(round(vs5_50$span_frac, digits=3)), log="y")


# Number of methods containing a given number of locals
meth_vars=count(method_info$num_locals)
plot(meth_vars, log="y", col=pal_col[2], cex=1.3,
	xaxs="i",
	xlim=c(0, 50),
	xlab="Local variables", ylab="Methods")

nl_mod=nls(log(freq) ~ a+b*x^c, data=meth_vars, subset=(x < 50),
				start=list(a=10, b=-2, c=0.9), trace=TRUE)
#   Estimate Std. Error t value Pr(>|t|)    
# a  14.9496     0.5788   25.83  < 2e-16 ***
# b  -3.3494     0.4551   -7.36 2.64e-09 ***
# c   0.3472     0.0243   14.29  < 2e-16 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Residual standard error: 0.1986 on 46 degrees of freedom

pred=predict(nl_mod)
lines(1:49, exp(pred), col=pal_col[1])

dev.copy(dev=png, file="locvar-meth_Java.png")
dev.off()


# Number of methods containing a given number of lines
meth_lines=count(method_info$num_lines)
plot(meth_lines, log="xy", col=pal_col[2], cex=1.3,
	xaxs="i",
	xlim=c(1, 500),
	xlab="LOC", ylab="Methods")

loc_mod=nls(log(freq) ~ a+b*x^c, data=meth_lines, subset=(x < 400),
                                start=list(a=10, b=-2, c=0.9), trace=TRUE)
#   Estimate Std. Error t value Pr(>|t|)    
# a  14.0078     0.5221   26.83  < 2e-16 ***
# b  -2.7704     0.3454   -8.02 2.03e-14 ***
# c   0.2802     0.0155   18.07  < 2e-16 ***
# ---
# Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
# 
# Residual standard error: 0.4262 on 319 degrees of freedom

xbounds=1:400
pred=predict(loc_mod, newdata=data.frame(x=xbounds))
lines(xbounds, exp(pred), col=pal_col[1])

dev.copy(dev=png, file="LOC-meth_Java.png")
dev.off()


xbounds=1:50

plot(method_info$num_locals, method_info$num_lines, log="xy", col=pal_col[2],
	xlim=c(1, 50), ylim=c(1, 1e3),
	xlab="Local variables", ylab="Lines")

nl_mod=glm(log(num_lines) ~ log(num_locals), data=method_info)
summary(nl_mod)
pred=predict(nl_mod, newdata=data.frame(num_locals=xbounds))
lines(xbounds, exp(pred), col=pal_col[1])

dev.copy(dev=png, file="locvar-LOC_Java.png")
dev.off()



smoothScatter(vs50$begin_ord, vs50$num_words)
plot(vs50$begin_ord, vs50$num_words)

plot(vs50$n_char, vs50$range, log="y")

plot(vs50$num_words, vs50$num_lines, log="y")
smoothScatter(vs50$num_words, log(vs50$num_lines))

vs50_mod=glm(log(range) ~ log(n_char)+log(num_words)+log(begin_ord), data=vs50)
summary(vs50_mod)

vs50_mod=glm(log(range) ~ log(n_char), data=vs50)
summary(vs50_mod)

vs50_mod=glm(log(span_frac) ~ log(n_char), data=vs50)
summary(vs50_mod)


nw_mod=glm(log(n_char) ~ log(begin_ord), data=vs50)
nw_mod=glm(log(num_words) ~ log(range)+log(begin_ord), data=vs50)
summary(nw_mod)


nw_mod=glm(log(n_char) ~ log(begin_ord), data=vs50)
nw_mod=glm(log(num_words) ~ log(span_frac)+log(begin_ord), data=vs50)
summary(nw_mod)


